#include "ofApp.h"

//--------------------------------------------------------------
void ofApp::setup(){
    camWidth = 320;
    camHeight = 240;
    //webcam
    camVideo.initGrabber(camWidth,camHeight);
    camPixels = new unsigned char[camWidth*camHeight*3];
    //negative video stream - size of the texture and of the pointer (char[])
    negativeImage.allocate(camWidth,camHeight,GL_RGB);
    negativePixels = new unsigned char[camWidth*camHeight*3];
    //glitch video stream
    glitchImage.allocate(camWidth,camHeight,GL_RGB);
    glitchPixels = new unsigned char[camWidth*camHeight*3];
}

//--------------------------------------------------------------
void ofApp::update(){
    camVideo.update();
    if(camVideo.isFrameNew()){
        //1. first we retrieve the pixels from the current frame
        //and store them in the camPixels pointer (char[])
        camPixels = camVideo.getPixels();
        //2. we read pixel by pixel of the camPixels pointer
        for(int i=0; i<camWidth*camHeight*3; i++){
        //we invert it and store it in  negativePixels
            negativePixels[i]=255-camPixels[i];
            //Build the glitch texture
            //RED if i is a multiple of 3 (ex: 0,3,6,9), we have a red pixel
            if(i%3==0){
                //we are leaving the red channel unchanged
                glitchPixels[i]=camPixels[i];
            }
            //GREEN
            if(i%3==1){
                if(i<80000){
                    //for green, we want random pixel values (noise)
                    glitchPixels[i]=ofRandom(255);
                    if(i%5==0){
                        glitchPixels[i]=255;
                    }
                }else{
                    glitchPixels[i]=0;
                }
            }
            //BLUE
            int pixelTrans = 60000;
            if(i%3==2 && i<(camWidth*camHeight*3-pixelTrans)){
                //for blue, we are translating pixels a certain amount (ex 60000) to the right
                glitchPixels[i+pixelTrans]=camPixels[i];
            }
        }
        //3. we load the pixels pointer (1D) into a 2D, RGB graphic structure
        negativeImage.loadData(negativePixels,camWidth,camHeight,GL_RGB);
        glitchImage.loadData(glitchPixels,camWidth,camHeight,GL_RGB);
    }
}

//--------------------------------------------------------------
void ofApp::draw(){
    camVideo.draw(20,20);
    negativeImage.draw(20+camWidth+20,20);
    glitchImage.draw(20+camWidth*2+40,20);
}

//--------------------------------------------------------------
void ofApp::keyPressed(int key){

}

//--------------------------------------------------------------
void ofApp::keyReleased(int key){

}

//--------------------------------------------------------------
void ofApp::mouseMoved(int x, int y){

}

//--------------------------------------------------------------
void ofApp::mouseDragged(int x, int y, int button){

}

//--------------------------------------------------------------
void ofApp::mousePressed(int x, int y, int button){

}

//--------------------------------------------------------------
void ofApp::mouseReleased(int x, int y, int button){

}

//--------------------------------------------------------------
void ofApp::windowResized(int w, int h){

}

//--------------------------------------------------------------
void ofApp::gotMessage(ofMessage msg){

}

//--------------------------------------------------------------
void ofApp::dragEvent(ofDragInfo dragInfo){ 

}